#! /usr/bin/env python
import random
import re
import sys

from version import __version__
import barf
import cfg

try:
	import redis
except:
	barf.Barf('ERR', 'Please install redis module for python.')
	sys.exit()


class scrib:
	"""
	Setting up gatekeeping and processing.
	"""
	chain_length = 2
	messages_to_generate = 5
	separator = '\x01'
	stop_word = '\x02'

	def __init__(self, interface, args):
		"""
		Here we'll load settings and set up modules.
		"""
		self.host = 'ataraxia.scndrls.net'
		self.port = '6379'
		self.tries = 0
		self.max_tries = 2
		self.cfg = cfg
		self.barf = barf.Barf

		self.redis = self.db_check(self.host, self.port)
		# Load interface settings
		self.interface = self.setcfg()
		self.interface.load(interface+'.cfg', args, suppress=True)
		self.prefix = interface

		# Load scrib settings
		self.settings = self.setcfg()
		self.settings.load('scrib.cfg', {
			'debug': 0,
			'version': __version__
		})
		self.barf('MSG', 'Scrib %s initialized' % __version__)

	def db_check(self, host='localhost', port='6379'):
		# Database check
		db = redis.Redis(host, port)
		# See if redis server is running
		try:
			ping = db.ping()
			if ping is not True:
				self.barf('ERR', 'Redis server is not running.')
				self.barf('ERR', 'Please install and run it, then try again.')
				sys.exit()
			else:
				self.barf('ACT', 'Connected to %s' % host)
				return db
		except Exception as e:
			self.barf('ERR', e.message.replace(host+':'+port, host))
			self.tries += 1
		
		while self.tries < self.max_tries:
			self.db_check()
		else:
			self.barf('ERR', 'No redis server found.')
			self.barf('ACT', 'Please make sure it is started or your host is correct.')
			sys.exit()

	"""
	Make some methods that give interfaces and plugins
	information in a controllable (and updatable) way.
	"""

	def setcfg(self):
		"""
		Simple wrapper for cfg.set()
		"""
		return self.cfg.set()

	def save_all(self):
		self.barf('SAV', 'Saving settings')
		self.settings.save()

	def shutdown(self):
		"""
		Shuts us the scrib down.
		"""
		msg = 'Shutting down. Goodbye!'
		try:
			if 'quitmsg' in self.interface._defaults:
				msg = self.interface._defaults['quitmsg']
		except:
			pass

		self.reply(msg, False)
		self.save_all()
		self.barf('MSG', msg)
		sys.exit()

	def reply(self, message, filter=True):
		if filter is False:
			return message

		messages = []

		for words in self.split(self.sanitize(message)):
			if self.settings.debug == 1:
				if words is not None:
					self.barf('DBG', words)
			key = self.separator.join(words[:-1])
			self.redis.sadd(self.make_key(self.prefix, key), words[-1])

			best_message = ''
			for i in range(self.messages_to_generate):
				generated = self.generate(seed=key)
				if len(generated) > len(best_message):
					best_message = generated

			if best_message:
				messages.append(best_message)

		if len(messages):
			return random.choice(messages)

	def make_key(self, prefix, k):
		return '-'.join((prefix, k))

	def sanitize(self, message):
		"""
		Sanitize the data.
		"""
		if self.settings.debug == 1:
			self.barf('DBG', 'Sanitizing %s ' % message)
		return re.sub('[\"\']', '', message)

	def split(self, message):
		"""
		Split message into word list!
		"""
		words = message.split()
		if len(words) > self.chain_length:
			words.append(self.stop_word)
			for i in range(len(words) - self.chain_length):
				yield words[i:i + self.chain_length + 1]

	def generate(self, seed):
		"""
		GENERATE ALL THE THINGS
		"""
		key = seed
		gen_words = []

		for i in xrange(self.interface.maxwords):
			words = key.split(self.separator)
			gen_words.append(words[0])
			next_word = self.redis.srandmember(self.make_key(key, self.prefix))
			if not next_word:
				break

			key = self.separator.join(words[1:] + [next_word])

		return ' '.join(gen_words)

	def format_error(self, e):
		self.barf('ERR', e)
